package OTPGen;

import javax.microedition.rms.*;
import java.io.*;

public class Account {
	public static final String[] hashNames = {"MD4", "MD5", "SHA1"};
	public static final int HASH_MD4 = 0;
	public static final int HASH_MD5 = 1;
	public static final int HASH_SHA1 = 2;

	String seed;
	int seq;
	int hash;
	int id; // record ID in store


	public Account(String newseed, int newseq, int newhash, int newid) {
		seed = new String(newseed);
		seq = newseq;
		hash = newhash;
		id = newid;
	}

	public Account() {
		this("", 0, HASH_MD4, 0);
	}

	public Account(Account a) {
		this(a.getSeed(), a.getSeq(), a.getHash(), a.getId());
	}

	public Account(int newid) {
		this("", 0, HASH_MD4, newid);
	}

	public String getLabel() {
		return (new String(hashNames[hash] + " " +
			Integer.toString(seq) + " " + seed)).toLowerCase();
	}

	public void save(String rsName)
			throws RecordStoreFullException, RecordStoreException {
		/* Data in a record:
		 * Seed: String
		 * Seq: int
		 * hash: int
		 */

		ByteArrayOutputStream stream = new ByteArrayOutputStream();
		DataOutputStream writer = new DataOutputStream(stream);

		try {
			writer.writeUTF(seed);
			writer.writeInt(seq);
			writer.writeInt(hash);

			writer.flush();
		}
		catch (IOException e) {
		}
		finally {
			RecordStore rs = RecordStore.openRecordStore(rsName,
						true);
			byte[] record = stream.toByteArray();

			if (id == 0) {	// new record
				id = rs.addRecord(record,0,record.length);
			} else {
				rs.setRecord(id, record, 0, record.length);
			}

			rs.closeRecordStore();
		}
	}


	public void delete(String rsName) {
		if (id != 0) {
			try {
				RecordStore rs = RecordStore.openRecordStore(rsName, true);
				rs.deleteRecord(id);
				rs.closeRecordStore();
			}
			catch (Exception e) {
				// System.out.println(e.toString());
			}
		}
	}

	public void setSeed(String newseed) {
		seed = newseed;
	}

	public String getSeed() {
		return seed;
	}

	public void setSeq(int newseq) {
		seq = newseq;
	}

	public int decSeq() {
		return --seq;
	}

	public int getSeq() {
		return seq;
	}

	public void setHash(int h) {
		hash = h;
	}

	public int getHash() {
		return hash;
	}

	public void setId(int newid) {
		id = newid;
	}

	public int getId() {
		return id;
	}
}
