package OTPGen;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;
import javax.microedition.rms.*;
import java.util.Vector;
import java.io.*;

public class ListAccounts extends List implements doAfterCommand
{
	public final static int OK = 0;
	public final static int EXIT = -1;
	public final static int EDIT = -2;
	public final static int ADD = -3;
	public final static int DEL = -4;
	public final static int COMPUTE = -5;
	public final static int ABOUT = -6;

	public final static String RSNAME = "accounts";

	int numAccounts;

	protected CommandListener owner;

	Account[] accounts;

	Command cmExit = new Command("Exit", Command.EXIT, 5);
	Command cmAdd = new Command("Add", Command.SCREEN, 0);
	Command cmEdit = new Command("Edit", Command.SCREEN, 1); 
	Command cmDel = new Command("Del", Command.SCREEN, 2);
	Command cmAbout = new Command("About", Command.SCREEN, 3);
	Command cmCompute = new Command("Compute", Command.SCREEN, 4);

	public ListAccounts(String name, int type, CommandListener l)
			throws RecordStoreException {
		super(name, type);

		owner = l;

		this.addCommand(cmAdd);
		this.addCommand(cmEdit);
		this.addCommand(cmDel);
		this.addCommand(cmAbout);
		this.addCommand(cmCompute);
		this.addCommand(cmExit);

		this.setCommandListener(owner);

		try {
			accounts = loadAccounts();
		}
		catch (RecordStoreException e) {
			accounts = new Account[0];
			throw(e);
		}

		rebuildList();

	}

	public int doCommand(Command cm) {
		if (cm == cmExit) {
			return EXIT;
		} else if (cm == cmDel) {
			int i = getSelectedIndex();
			if (i >= 0) {
				accounts[i].delete(RSNAME);
				this.delete(i);
			}
			return DEL;
		} else if (cm == cmEdit) {
			return EDIT;
		} else if (cm == cmAdd) {
			return ADD;
		} else if ((cm == cmCompute) || (cm == List.SELECT_COMMAND)) {
			return COMPUTE;
		} else if (cm == cmAbout) {
			return ABOUT;
		}
		return OK;
	}

	public Account getCurrentAccount() {
		if (getSelectedIndex() >= 0) {
			return accounts[getSelectedIndex()];
		} else {
			return null;
		}
	}

	private Account[] fakeLoad() {
		Account[] accs = new Account[1];

		accs[0] = new Account("alpha1", 1, Account.HASH_SHA1, 0);

		numAccounts = 1;
		return accs;
	}

	private Account[] loadAccounts() throws RecordStoreException {
		/* Data in a record:
		 * Seed: String	(max. 16)
		 * Seq: int
		 * hash: int
		 */

		RecordStore rs = RecordStore.openRecordStore(RSNAME, true);
		RecordEnumeration renum = rs.enumerateRecords(null,
				new AccountComparator(), false);

		byte[] record = new byte[24];
		ByteArrayInputStream stream = new ByteArrayInputStream(record);
		DataInputStream reader = new DataInputStream(stream);

		Account[] accs = new Account[renum.numRecords()];
		int i = 0;
		int id = 0;

		while(renum.hasNextElement()) {
			id = renum.nextRecordId();
			rs.getRecord(id, record, 0);

			Account a = new Account(id);

			try {
				a.setSeed(reader.readUTF());
				a.setSeq(reader.readInt());
				a.setHash(reader.readInt());

				reader.reset();
				stream.reset();
			}
			catch (IOException e) {
				// debug
				// System.out.println(e.toString());
			}
			finally {
				accs[i++] = a;
			}
		}

		numAccounts = i;
		renum.destroy();
		rs.closeRecordStore();

		return accs;
	}

	private void rebuildList() {
                for (int i = 0; i < numAccounts; i++) {
                        if (accounts[i] != null) {	// FIXME
				this.append(accounts[i].getLabel(), null);
                        }
                }
        }
}
