package OTPGen;

/* A simplistic MD4 implementation that works on octets only */
public class MD4 implements HASH {
	protected int a, b, c, d;
	protected long count;

	public MD4() {
		reset();
	}

	public void reset() {
		// java is big endian
		a = 0x67452301;
		b = 0xefcdab89;
		c = 0x98badcfe;
		d = 0x10325476;
		count = 0;
	}

	/* reads no more that 64 bytes */
	public void transformBlock(byte[] input, int start) {
		byte[] in;

		count += (input.length-start) * 8;

		if ((input.length-start) < 64) {
			// FIXME padding
			in = pad(input);
		} else {
			in = input;
		}

		int[] X = new int[16];
		// copy and reverse endianness
		for(int b = 0, i = start; i < 16+start; i++) {
			X[i] = (in[b++] & 0xff) | ((in[b++] & 0xff) << 8) |
				((in[b++]&0xff) << 16) | ((in[b++]&0xff) << 24);
		}

		this.transformations(X);
	}


	public byte[] getResult() {
		byte[] digest = {
			(byte) a, (byte) (a >>> 8), (byte) (a >>> 16),
				(byte) (a >>> 24),
			(byte) b, (byte) (b >>> 8), (byte) (b >>> 16),
				(byte) (b >>> 24),
			(byte) c, (byte) (c >>> 8), (byte) (c >>> 16),
				(byte) (c >>> 24),
			(byte) d, (byte) (d >>> 8), (byte) (d >>> 16),
				(byte) (d >>> 24)
		};
		return digest;
	}

	public byte[] foldTo64() {
		byte[] digest = getResult();
		byte[] folded = new byte[8];

		for (int i = 0; i < 8; i++) {
			digest[i] ^= digest[i+8];
			folded[i] = digest[i];
		}

		return folded;
	}
			
	protected synchronized void transformations(int[] X) {
		int i;
		int aa = a;
		int bb = b;
		int cc = c;
		int dd = d;

		// round 1!
		for(i = 0; i < 4; i++) {
			int _k = 4*i;
			aa += ((bb & cc) | ((~bb) & dd)) + X[_k];
			aa = (aa << 3) | (aa >>> -3);	// shiftrot
			dd += ((aa & bb) | ((~aa) & cc)) + X[_k+1];
			dd = (dd << 7) | (dd >>> -7);
			cc += ((dd & aa) | ((~dd) & bb)) + X[_k+2];
			cc = (cc << 11) | (cc >>> -11);
			bb += ((cc & dd) | ((~cc) & aa)) + X[_k+3];
			bb = (bb << 19) | (bb >>> -19);
		}

		// round 2!
		for(i = 0; i < 4; i++) {
			aa += ((bb & (cc | dd)) | (cc & dd)) + X[i]
					+ 0x5a827999;
			aa = (aa << 3) | (aa >>> -3);
			dd += ((aa & (bb | cc)) | (bb & cc)) + X[i+4]
					+ 0x5a827999;
			dd = (dd << 5) | (dd >>> -5);
			cc += ((dd & (aa | bb)) | (aa & bb)) + X[i+8]
					+ 0x5a827999;
			cc = (cc << 9) | (cc >>> -9);
			bb += ((cc & (dd | aa)) | (dd & aa)) + X[i+12]
					+ 0x5a827999;
			bb = (bb << 13) | (bb >>> -13);
		}

		// round 3! almost there...
		aa += (bb ^ cc ^ dd) + X[0] + 0x6ed9eba1;
		aa = (aa << 3) | (aa >>> -3);
		dd += (aa ^ bb ^ cc) + X[8] + 0x6ed9eba1;
		dd = (dd << 9) | (dd >>> -9);
		cc += (dd ^ aa ^ bb) + X[4] + 0x6ed9eba1;
		cc = (cc << 11) | (cc >>> -11);
		bb += (cc ^ dd ^ aa) + X[12] + 0x6ed9eba1;
		bb = (bb << 15) | (bb >>> -15);

		aa += (bb ^ cc ^ dd) + X[2] + 0x6ed9eba1;
		aa = (aa << 3) | (aa >>> -3);
		dd += (aa ^ bb ^ cc) + X[10] + 0x6ed9eba1;
		dd = (dd << 9) | (dd >>> -9);
		cc += (dd ^ aa ^ bb) + X[6] + 0x6ed9eba1;
		cc = (cc << 11) | (cc >>> -11);
		bb += (cc ^ dd ^ aa) + X[14] + 0x6ed9eba1;
		bb = (bb << 15) | (bb >>> -15);

		aa += (bb ^ cc ^ dd) + X[1] + 0x6ed9eba1;
		aa = (aa << 3) | (aa >>> -3);
		dd += (aa ^ bb ^ cc) + X[9] + 0x6ed9eba1;
		dd = (dd << 9) | (dd >>> -9);
		cc += (dd ^ aa ^ bb) + X[5] + 0x6ed9eba1;
		cc = (cc << 11) | (cc >>> -11);
		bb += (cc ^ dd ^ aa) + X[13] + 0x6ed9eba1;
		bb = (bb << 15) | (bb >>> -15);

		aa += (bb ^ cc ^ dd) + X[3] + 0x6ed9eba1;
		aa = (aa << 3) | (aa >>> -3);
		dd += (aa ^ bb ^ cc) + X[11] + 0x6ed9eba1;
		dd = (dd << 9) | (dd >>> -9);
		cc += (dd ^ aa ^ bb) + X[7] + 0x6ed9eba1;
		cc = (cc << 11) | (cc >>> -11);
		bb += (cc ^ dd ^ aa) + X[15] + 0x6ed9eba1;
		bb = (bb << 15) | (bb >>> -15);

		a += aa; b += bb; c += cc; d += dd;
	}

	/* pad input that is shorter that 64 bytes */
	protected byte[] pad(byte[] in) {
		byte[] padded = new byte[64];

		if (in.length < 64) {
			for(int i = 0; i < in.length; i++) {
				padded[i] = in[i];
			}
			padded[in.length] = (byte)0x80;	
			for(int i = in.length + 1; i < 56; i++) {
				padded[i] = (byte)0x00;  // FIXME: make it faster
			}
			// append length
			padded[56] = (byte)  count;
			padded[57] = (byte) (count >>>  8);
			padded[58] = (byte) (count >>> 16);
			padded[59] = (byte) (count >>> 24);
			padded[60] = (byte) (count >>> 32);
			padded[61] = (byte) (count >>> 40);
			padded[62] = (byte) (count >>> 48);
			padded[63] = (byte) (count >>> 56);
		}
		return padded;
	}
}
