package OTPGen;

public class SHA1 extends MD4 implements HASH {
	protected int e;

	public SHA1() {
		reset();
	}

	public void reset() {
		a = 0x67452301;
		b = 0xefcdab89;
		c = 0x98badcfe;
		d = 0x10325476;
		e = 0xc3d2e1f0;

		count = 0;
	}

	protected synchronized void transformations(int[] X) {
		int aa = a;
		int bb = b;
		int cc = c;
		int dd = d;
		int ee = e;

		int[] w = new int[80];
		int temp, t;

		for(t = 0; t < 16; t++) {
			w[t] =  ((X[t] & 0xff) << 24) | ((X[t] & 0xff00) << 8) |
				((X[t] & 0xff0000) >>> 8) |
				((X[t] & 0xff000000) >>> 24);
		}
		
		for(t = 16; t < 80; t++) {
			temp = w[t-3] ^ w[t-8] ^ w[t-14] ^ w[t-16];
			w[t] = (temp << 1) | (temp >>> -1);
		}

		for(t = 0; t < 20; t++) {
			temp = ((aa << 5) | (aa >>> -5)) + w[t] + ee +
				((bb & cc) | (~bb & dd)) + 0x5a827999;
			ee = dd;
			dd = cc;
			cc = (bb << 30) | (bb >>> -30);
			bb = aa;
			aa = temp;
		}

		for(t = 20; t < 40; t++) {
			temp = ((aa << 5) | (aa >>> -5)) + w[t] + ee +
				(bb ^ cc ^ dd) + 0x6ed9eba1;
			ee = dd;
			dd = cc;
			cc = (bb << 30) | (bb >>> -30);
			bb = aa;
			aa = temp;
		}

		for(t = 40; t < 60; t++) {
			temp = ((aa << 5) | (aa >>> -5)) + w[t] + ee +
				((bb & cc) | (bb & dd) | (cc & dd))+0x8f1bbcdc;
			ee = dd;
			dd = cc;
			cc = (bb << 30) | (bb >>> -30);
			bb = aa;
			aa = temp;
		}

		for(t = 60; t < 80; t++) {
			temp = ((aa << 5) | (aa >>> -5)) + w[t] + ee +
				(bb ^ cc ^ dd) + 0xca62c1d6;
			ee = dd;
			dd = cc;
			cc = (bb << 30) | (bb >>> -30);
			bb = aa;
			aa = temp;
		}

		a += aa; b += bb; c += cc; d += dd; e += ee;
	}

	public byte[] getResult() {
		byte[] digest = {
			(byte)(a >>> 24), (byte)(a >>> 16), (byte)(a >>> 8),
				(byte) a,
			(byte)(b >>> 24), (byte)(b >>> 16), (byte)(b >>> 8),
				(byte) b,
			(byte)(c >>> 24), (byte)(c >>> 16), (byte)(c >>> 8),
				(byte) c,
			(byte)(d >>> 24), (byte)(d >>> 16), (byte)(d >>> 8),
				(byte) d,
			(byte)(e >>> 24), (byte)(e >>> 16), (byte)(e >>> 8),
				(byte) e
		};
		return digest;
	}

	public byte[] foldTo64() {
		byte[] folded = new byte[8];
		int aa;
		int bb;

		aa = a ^ c ^ e;
		bb = b ^ d;

		for(int i = 0; i < 4; i++)
			folded[i] = (byte)(aa >>> (i*8));

		for(int i = 0; i < 4; i++)
			folded[i+4] = (byte)(bb >>> (i*8));

		return folded;
	}

	protected byte[] pad(byte[] in) {
		byte[] padded = new byte[64];

		if (in.length < 64) {
			int i;
			for(i =0; i < in.length; i++)
				padded[i] = in[i];

			padded[in.length] = (byte)0x80;
			for(i = in.length + 1; i < 56; i++)
				padded[i] = (byte)0x00;

			// append length
			padded[56] = (byte) (count >>> 56);
			padded[57] = (byte) (count >>> 48);
			padded[58] = (byte) (count >>> 40);
			padded[59] = (byte) (count >>> 32);
			padded[60] = (byte) (count >>> 24);
			padded[61] = (byte) (count >>> 16);
			padded[62] = (byte) (count >>>  8);
			padded[63] = (byte)  count;
		}
		return padded;
	}
}
	
